(function() {
  var XhrPollingReceiver, XhrStreamingReceiver, transport, utils;
  var __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
    for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
    function ctor() { this.constructor = child; }
    ctor.prototype = parent.prototype;
    child.prototype = new ctor;
    child.__super__ = parent.prototype;
    return child;
  }, __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  transport = require('./transport');
  utils = require('./utils');
  XhrPollingReceiver = (function() {
    __extends(XhrPollingReceiver, transport.ResponseReceiver);
    function XhrPollingReceiver() {
      XhrPollingReceiver.__super__.constructor.apply(this, arguments);
    }
    XhrPollingReceiver.prototype.protocol = "xhr";
    XhrPollingReceiver.prototype.doSendFrame = function(payload) {
      var ondrain, r, write;
      if (this.session) {
        this.session.unregister();
      }
      if (payload !== 'o') {
        r = XhrPollingReceiver.__super__.doSendFrame.call(this, payload + '\n');
        this.response.end();
        return r;
      } else {
        write = __bind(function(payload) {
          try {
            return this.response.write(payload + '\n');
          } catch (x) {

          }
          return true;
        }, this);
        ondrain = __bind(function() {
          this.response.connection.removeListener('drain', ondrain);
          return utils.timeout_chain([
            [
              150, __bind(function() {
                return write("o");
              }, this)
            ], [
              150, __bind(function() {
                write("");
                return this.response.end();
              }, this)
            ]
          ]);
        }, this);
        r = write(Array(2048).join('h') + '\n');
        if (r === false) {
          this.response.connection.addListener('drain', ondrain);
        } else {
          ondrain();
        }
        return true;
      }
    };
    XhrPollingReceiver.prototype.doKeepalive = function() {
      return this.doSendFrame("h");
    };
    return XhrPollingReceiver;
  })();
  XhrStreamingReceiver = (function() {
    __extends(XhrStreamingReceiver, transport.ResponseReceiver);
    XhrStreamingReceiver.prototype.protocol = "xhr";
    function XhrStreamingReceiver() {
      this.send_bytes = 0;
      XhrStreamingReceiver.__super__.constructor.apply(this, arguments);
    }
    XhrStreamingReceiver.prototype.doSendFrame = function(payload) {
      var r;
      this.send_bytes += payload.length + 1;
      if (this.send_bytes > 128 * 1024) {
        if (this.session) {
          this.session.unregister();
        }
      }
      r = XhrStreamingReceiver.__super__.doSendFrame.call(this, payload + '\n');
      if (this.send_bytes > 128 * 1024) {
        this.response.end();
      }
      return r;
    };
    XhrStreamingReceiver.prototype.doKeepalive = function() {
      return this.doSendFrame("h");
    };
    return XhrStreamingReceiver;
  })();
  exports.app = {
    xhr_poll: function(req, res, _, next_filter) {
      var session;
      res.setHeader('Content-Type', 'application/javascript; charset=UTF-8');
      res.writeHead(200);
      session = transport.Session.bySessionIdOrNew(req.session, req.sockjs_server);
      session.register(new XhrPollingReceiver(res));
      return true;
    },
    xhr_options: function(req, res) {
      res.statusCode = 204;
      res.setHeader('Allow', 'OPTIONS, POST');
      res.setHeader('Access-Control-Max-Age', res.cache_for);
      return '';
    },
    xhr_send: function(req, res, data) {
      var d, jsonp, message, _i, _len;
      if (!data) {
        throw {
          status: 500,
          message: 'payload expected'
        };
      }
      d = JSON.parse(data);
      if (!d || d.__proto__.constructor !== Array) {
        throw {
          status: 500,
          message: 'payload expected'
        };
      }
      jsonp = transport.Session.bySessionId(req.session);
      if (!jsonp) {
        throw {
          status: 404
        };
      }
      for (_i = 0, _len = d.length; _i < _len; _i++) {
        message = d[_i];
        jsonp.didMessage(message);
      }
      res.writeHead(200);
      res.end();
      return true;
    },
    xhr_cors: function(req, res, content) {
      var headers, origin;
      origin = req.headers['origin'] || '*';
      res.setHeader('Access-Control-Allow-Origin', origin);
      headers = req.headers['access-control-request-headers'];
      if (headers) {
        res.setHeader('Access-Control-Allow-Headers', headers);
      }
      res.setHeader('Access-Control-Allow-Credentials', 'true');
      return content;
    },
    xhr_streaming: function(req, res, _, next_filter) {
      var session;
      res.setHeader('Content-Type', 'application/javascript; charset=UTF-8');
      res.writeHead(200);
      res.write(Array(2048).join('h') + '\n');
      session = transport.Session.bySessionIdOrNew(req.session, req.sockjs_server);
      session.register(new XhrStreamingReceiver(res));
      return true;
    }
  };
}).call(this);
